@@ -19,9 +19,9 @@
  *
  */
 
-#include "nvtop/interface.h"
 #include "nvtop/common.h"
 #include "nvtop/extract_gpuinfo_common.h"
+#include "nvtop/interface.h"
 #include "nvtop/interface_common.h"
 #include "nvtop/interface_internal_common.h"
 #include "nvtop/interface_layout_selection.h"
@@ -43,9 +43,9 @@
 #include <unistd.h>
 
 static unsigned int sizeof_device_field[device_field_count] = {
-    [device_name] = 11,       [device_fan_speed] = 11,  [device_temperature] = 10,
-    [device_power] = 15,      [device_clock] = 11,      [device_pcie] = 46,
-    [device_shadercores] = 7, [device_l2features] = 11, [device_execengines] = 11,
+    [device_name] = 11,       [device_fan_speed] = 11,   [device_temperature] = 10, [device_power] = 15,
+    [device_clock] = 11,      [device_mem_clock] = 12,   [device_pcie] = 46,        [device_shadercores] = 7,
+    [device_l2features] = 11, [device_execengines] = 11,
 };
 
 static unsigned int sizeof_process_field[process_field_count] = {
@@ -74,22 +74,24 @@ static void alloc_device_window(unsigned int start_row, unsigned int start_col,
   dwin->gpu_clock_info = newwin(1, sizeof_device_field[device_clock], start_row + 1, start_col);
   if (dwin->gpu_clock_info == NULL)
     goto alloc_error;
-  dwin->mem_clock_info = newwin(1, sizeof_device_field[device_clock], start_row + 1,
+  dwin->mem_clock_info = newwin(1, sizeof_device_field[device_mem_clock], start_row + 1,
                                 start_col + spacer + sizeof_device_field[device_clock]);
   if (dwin->mem_clock_info == NULL)
     goto alloc_error;
-  dwin->temperature = newwin(1, sizeof_device_field[device_temperature], start_row + 1,
-                             start_col + spacer * 2 + sizeof_device_field[device_clock] * 2);
+  dwin->temperature =
+      newwin(1, sizeof_device_field[device_temperature], start_row + 1,
+             start_col + spacer * 2 + sizeof_device_field[device_clock] + sizeof_device_field[device_mem_clock]);
   if (dwin->temperature == NULL)
     goto alloc_error;
-  dwin->fan_speed =
-      newwin(1, sizeof_device_field[device_fan_speed], start_row + 1,
-             start_col + spacer * 3 + sizeof_device_field[device_clock] * 2 + sizeof_device_field[device_temperature]);
+  dwin->fan_speed = newwin(1, sizeof_device_field[device_fan_speed], start_row + 1,
+                           start_col + spacer * 3 + sizeof_device_field[device_clock] +
+                               sizeof_device_field[device_mem_clock] + sizeof_device_field[device_temperature]);
   if (dwin->fan_speed == NULL)
     goto alloc_error;
-  dwin->power_info = newwin(1, sizeof_device_field[device_power], start_row + 1,
-                            start_col + spacer * 4 + sizeof_device_field[device_clock] * 2 +
-                                sizeof_device_field[device_temperature] + sizeof_device_field[device_fan_speed]);
+  dwin->power_info =
+      newwin(1, sizeof_device_field[device_power], start_row + 1,
+             start_col + spacer * 4 + sizeof_device_field[device_clock] + sizeof_device_field[device_mem_clock] +
+                 sizeof_device_field[device_temperature] + sizeof_device_field[device_fan_speed]);
   if (dwin->power_info == NULL)
     goto alloc_error;
 
@@ -341,9 +343,9 @@ static void alloc_plot_window(unsigned devices_count, struct window_position *pl
 
 static unsigned device_length(void) {
   return max(sizeof_device_field[device_name] + sizeof_device_field[device_pcie] + 1,
-
-             2 * sizeof_device_field[device_clock] + sizeof_device_field[device_temperature] +
-                 sizeof_device_field[device_fan_speed] + sizeof_device_field[device_power] + 4);
+             sizeof_device_field[device_clock] + sizeof_device_field[device_mem_clock] +
+                 sizeof_device_field[device_temperature] + sizeof_device_field[device_fan_speed] +
+                 sizeof_device_field[device_power] + 5);
 }
 
 static pid_t nvtop_pid;
@@ -491,6 +493,23 @@ static void draw_percentage_meter(WINDOW *win, const char *prelude, unsigned int
   wnoutrefresh(win);
 }
 
+// Draw percentage with a yellow highlight percentage (yellow percentage <= new_percentage)
+static void draw_percentage_meter_with_yellow_highlight(WINDOW *win, const char *prelude, unsigned int new_percentage,
+                                                        unsigned int yellow_percentage,
+                                                        const char inside_braces_right[1024]) {
+  draw_percentage_meter(win, prelude, new_percentage, inside_braces_right);
+  if (yellow_percentage > new_percentage)
+    yellow_percentage = new_percentage;
+  int rows, cols;
+  getmaxyx(win, rows, cols);
+  (void)rows;
+  size_t size_prelude = strlen(prelude);
+  int between_sbraces = cols - size_prelude - 2;
+  float usage = round((float)between_sbraces * yellow_percentage / 100.f);
+  mvwchgat(win, 0, size_prelude + 1, (int)usage, 0, yellow_color, NULL);
+  wnoutrefresh(win);
+}
+
 static const char *memory_prefix[] = {" B", "Ki", "Mi", "Gi", "Ti", "Pi"};
 
 static void draw_temp_color(WINDOW *win, unsigned int temp, unsigned int temp_slowdown, bool celsius) {
@@ -683,8 +702,15 @@ static void draw_devices(struct list_head *devices, struct nvtop_interface *inte
         draw_percentage_meter(decode_win, "DEC", rate, buff);
     }
     if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, gpu_util_rate)) {
-      snprintf(buff, 1024, "%u%%", device->dynamic_info.gpu_util_rate);
-      draw_percentage_meter(gpu_util_win, "GPU", device->dynamic_info.gpu_util_rate, buff);
+      if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, effective_load_rate)) {
+        snprintf(buff, 1024, "%u%%(eff %u%%)", device->dynamic_info.gpu_util_rate,
+                 device->dynamic_info.effective_load_rate);
+        draw_percentage_meter_with_yellow_highlight(gpu_util_win, "GPU", device->dynamic_info.gpu_util_rate,
+                                                    device->dynamic_info.effective_load_rate, buff);
+      } else {
+        snprintf(buff, 1024, "%u%%", device->dynamic_info.gpu_util_rate);
+        draw_percentage_meter(gpu_util_win, "GPU", device->dynamic_info.gpu_util_rate, buff);
+      }
     } else {
       snprintf(buff, 1024, "N/A");
       draw_percentage_meter(gpu_util_win, "GPU", 0, buff);
@@ -1688,6 +1714,11 @@ void save_current_data_to_ring(struct list_head *devices, struct nvtop_interface
             data_val = device->dynamic_info.mem_clock_speed * 100 / device->dynamic_info.mem_clock_speed_max;
           }
           break;
+        case plot_effective_load_rate:
+          if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, effective_load_rate)) {
+            data_val = device->dynamic_info.effective_load_rate;
+          }
+          break;
         case plot_information_count:
           break;
         }
@@ -1716,7 +1747,7 @@ static unsigned populate_plot_data_from_ring_buffer(const struct nvtop_interface
   assert(total_to_draw > 0);
   assert(size_data_buff % total_to_draw == 0);
   unsigned max_data_to_copy = size_data_buff / total_to_draw;
-  double(*data_split)[total_to_draw] = (double(*)[total_to_draw])data;
+  double (*data_split)[total_to_draw] = (double (*)[total_to_draw])data;
 
   unsigned in_processing = 0;
   for (unsigned i = 0; i < plot_win->num_devices_to_plot; ++i) {
@@ -1754,6 +1785,9 @@ static unsigned populate_plot_data_from_ring_buffer(const struct nvtop_interface
         case plot_gpu_mem_clock_rate:
           snprintf(plot_legend[in_processing], PLOT_MAX_LEGEND_SIZE, "GPU%u mem clock%%", dev_id);
           break;
+        case plot_effective_load_rate:
+          snprintf(plot_legend[in_processing], PLOT_MAX_LEGEND_SIZE, "GPU%u eff. load%%", dev_id);
+          break;
         case plot_information_count:
           break;
         }
@@ -1942,6 +1976,10 @@ void interface_key(int keyId, struct nvtop_interface *interface) {
   case 27:
     interface->process.option_window.state = nvtop_option_state_hidden;
     break;
+  case KEY_F(5):
+  case 12: // Ctrl+L
+    update_window_size_to_terminal_size(interface);
+    break;
   default:
     break;
   }
@@ -2074,6 +2112,9 @@ void print_snapshot(struct list_head *devices, bool use_fahrenheit_option) {
     const char *power_field = "power_draw";
     const char *gpu_util_field = "gpu_util";
     const char *mem_util_field = "mem_util";
+    const char *mem_total_field = "mem_total";
+    const char *mem_used_field = "mem_used";
+    const char *mem_free_field = "mem_free";
 
     printf("%s{\n", indent_level_two);
 
@@ -2137,6 +2178,21 @@ void print_snapshot(struct list_head *devices, bool use_fahrenheit_option) {
       printf("%s\"%s\": \"%u%%\"\n", indent_level_four, mem_util_field, device->dynamic_info.mem_util_rate);
     else
       printf("%s\"%s\": null\n", indent_level_four, mem_util_field);
+    // Memory Total
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, total_memory))
+      printf("%s\"%s\": \"%llu\"\n", indent_level_four, mem_total_field, device->dynamic_info.total_memory);
+    else
+      printf("%s\"%s\": null\n", indent_level_four, mem_total_field);
+    // Memory Used
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, used_memory))
+      printf("%s\"%s\": \"%llu\"\n", indent_level_four, mem_used_field, device->dynamic_info.used_memory);
+    else
+      printf("%s\"%s\": null\n", indent_level_four, mem_used_field);
+    // Memory Available
+    if (GPUINFO_DYNAMIC_FIELD_VALID(&device->dynamic_info, free_memory))
+      printf("%s\"%s\": \"%llu\"\n", indent_level_four, mem_free_field, device->dynamic_info.free_memory);
+    else
+      printf("%s\"%s\": null\n", indent_level_four, mem_free_field);
 
     if (device->list.next == devices)
       printf("%s}\n", indent_level_two);
